"""Functions to import assets into a PDF project."""

from typing import List

from .base import BaseAbstractAssetImporter, BatchParams, ContentBatchImporter
from .types import AssetLike


class PdfDataImporter(BaseAbstractAssetImporter):
    """Class for importing data into a PDF project."""

    def import_assets(self, assets: List[AssetLike]):
        """Import PDF assets into Kili."""
        self._check_upload_is_allowed(assets)
        is_hosted = self.is_hosted_content(assets)
        assets = self.filter_duplicate_external_ids(assets)

        batch_params = BatchParams(is_hosted=is_hosted, is_asynchronous=False)
        batch_importer = ContentBatchImporter(
            self.kili, self.project_params, batch_params, self.pbar
        )
        return self.import_assets_by_batch(assets, batch_importer)



"""GraphQL payload data mappers for cloud storage operations."""

from typing import Dict

from kili.domain.cloud_storage import DataConnectionFilters, DataIntegrationFilters

from .types import (
    AddDataConnectionKiliAPIGatewayInput,
    DataConnectionComputeDifferencesKiliAPIGatewayInput,
)


def data_integration_where_mapper(filters: DataIntegrationFilters) -> Dict:
    """Build the GraphQL DataIntegrationWhere variable to be sent in an operation."""
    return {
        "status": filters.status,
        "id": filters.id,
        "name": filters.name,
        "organizationId": filters.organization_id,
        "platform": filters.platform,
    }


def data_connection_where_mapper(filters: DataConnectionFilters) -> Dict:
    """Build the GraphQL DataConnectionsWhere variable to be sent in an operation."""
    return {
        "projectId": filters.project_id,
        "integrationId": filters.integration_id,
    }


def add_data_connection_data_mapper(data: AddDataConnectionKiliAPIGatewayInput) -> Dict:
    """Build the GraphQL DataConnectionInput variable to be sent in an operation."""
    return {
        "projectId": data.project_id,
        "integrationId": data.integration_id,
        "isChecking": data.is_checking,
        "lastChecked": data.last_checked.isoformat(sep="T", timespec="milliseconds") + "Z",
        "selectedFolders": data.selected_folders,
    }


def compute_data_connection_difference_data_mapper(
    data: DataConnectionComputeDifferencesKiliAPIGatewayInput,
) -> Dict:
    """Build the GraphQL DataConnectionComputeDifferencesPayload variable."""
    return {
        "blobPaths": data.blob_paths,
        "warnings": data.warnings,
    }



"""Project domain."""

from dataclasses import dataclass
from typing import TYPE_CHECKING, Literal, NewType, Optional

from .types import ListOrTuple

if TYPE_CHECKING:
    from .tag import TagId

ProjectId = NewType("ProjectId", str)
InputType = Literal["IMAGE", "PDF", "TEXT", "VIDEO"]

ComplianceTag = Literal["PHI", "PII"]


@dataclass
# pylint: disable=too-many-instance-attributes
class ProjectFilters:
    """Project filters for running a project search."""

    id: Optional[ProjectId]  # noqa: A003
    archived: Optional[bool] = None
    search_query: Optional[str] = None
    should_relaunch_kpi_computation: Optional[bool] = None
    starred: Optional[bool] = None
    updated_at_gte: Optional[str] = None
    updated_at_lte: Optional[str] = None
    created_at_gte: Optional[str] = None
    created_at_lte: Optional[str] = None
    tag_ids: Optional[ListOrTuple["TagId"]] = None



"""Formatters for assets retrieved from Kili API."""

import json
from typing import Dict

from kili.domain.types import ListOrTuple


def load_asset_json_fields(asset: Dict, fields: ListOrTuple[str]) -> Dict:
    """Load json fields of an asset."""
    if "jsonMetadata" in fields:
        asset["jsonMetadata"] = json.loads(asset.get("jsonMetadata", "{}"))

    if "labels.jsonResponse" in fields:
        asset_labels = asset.get("labels", [])
        for label in asset_labels:
            label["jsonResponse"] = json.loads(label["jsonResponse"])

    if "latestLabel.jsonResponse" in fields and asset.get("latestLabel") is not None:
        asset["latestLabel"]["jsonResponse"] = json.loads(asset["latestLabel"]["jsonResponse"])

    return asset



"""Types for the Cloud storage related Kili API gateway functions."""

from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from kili.domain.cloud_storage import DataIntegrationId
from kili.domain.project import ProjectId


@dataclass
class AddDataConnectionKiliAPIGatewayInput:
    """Add data connection input data for Kili API Gateway."""

    is_checking: bool
    integration_id: DataIntegrationId
    last_checked: datetime
    project_id: ProjectId
    selected_folders: Optional[List[str]]


@dataclass
class DataConnectionComputeDifferencesKiliAPIGatewayInput:
    """Data connection compute differences input data for Kili API Gateway."""

    blob_paths: List[str]
    warnings: List[str]
